@@ -10,6 +10,7 @@ use Getopt::Long;
 use Test::More;
 use Data::Dumper;
 use File::Temp qw(tempfile);
+use List::Util qw(shuffle);
 
 use base qw(Exporter);
 our @EXPORT = qw(
@@ -21,10 +22,9 @@ our @EXPORT = qw(
 use Devel::NYTProf::Data;
 use Devel::NYTProf::Reader;
 use Devel::NYTProf::Util qw(strip_prefix_from_paths html_safe_filename);
+use Devel::NYTProf::Run qw(perl_command_words);
 
-
-my $this_perl = $^X;
-$this_perl .= $Config{_exe} if $^O ne 'VMS' and $this_perl !~ m/$Config{_exe}$/i;
+my $diff_opts = ($Config{osname} eq 'MSWin32') ? '-c' : '-u';
 
 my %opts = (
     one          => $ENV{NYTPROF_TEST_ONE},
@@ -69,7 +69,12 @@ chdir('t') if -d 't';
 if (-d '../blib') {
     unshift @INC, '../blib/arch', '../blib/lib';
 }
-my $bindir      = (grep {-d} qw(./blib/script ../blib/script))[0];
+my $bindir      = (grep {-d} qw(./blib/script ../blib/script))[0] || do {
+    my $bin = (grep {-d} qw(./bin ../bin))[0]
+        or die "Can't find scripts";
+    warn "Couldn't find blib/script directory, so using $bin";
+    $bin;
+};
 my $nytprofcsv  = "$bindir/nytprofcsv";
 my $nytprofhtml = "$bindir/nytprofhtml";
 my $nytprofmerge= "$bindir/nytprofmerge";
@@ -122,10 +127,12 @@ sub mk_opt_combinations {
             }
         }
     }
+    @opt_combinations = shuffle @opt_combinations;
     return \@opt_combinations;
 }
 
 my %env_influence;
+my %env_failed;
 
 
 sub do_foreach_opt_combination {
@@ -156,6 +163,7 @@ sub do_foreach_opt_combination {
         # record what env settings may have influenced the failure
         ++$env_influence{$_}{$env->{$_}}{$failed ? 'fail' : 'pass'}
             for keys %$env;
+        $env_failed{ $ENV{NYTPROF} } = $failed;
     }
 }
 
@@ -180,13 +188,16 @@ sub report_env_influence {
         push @env_influence, sprintf "%15s: %s\n", $envvar,
             join ', ', map { "$_ => $variants->{$_}" } sort keys %$variants;
     }
-    %env_influence = ();
-
     if (@env_influence and not defined wantarray) {
+        push @env_influence, sprintf "%s with %s\n",
+                $env_failed{$_} ? 'FAILED' : 'Passed', $_
+            for sort keys %env_failed;
+
         diag "SUMMARY: Breakdown of $tag test failures by option settings:";
         diag $_ for @env_influence;
     }
 
+    %env_influence = ();
     return @env_influence;
 }
 
@@ -197,6 +208,13 @@ sub run_test_group {
     my $extra_test_code  = $rtg_opts->{extra_test_code};
     my $extra_test_count = $rtg_opts->{extra_test_count} || 0;
     my $extra_options    = $rtg_opts->{extra_options};
+    if ($ENV{NYTPROF_TEST_NOEXTRA}) {
+        diag "NYTPROF_TEST_NOEXTRA - skipping $extra_test_count extra tests"
+            if $extra_test_count;
+        $extra_test_code = undef;
+        $extra_test_count = 0;
+        $extra_options = {};
+    }
 
     # obtain group from file name
     my $group;
@@ -282,7 +300,9 @@ sub run_test {
             or die "Profiling $test failed\n";
 
         if ($opts{html}) {
-            my $cmd = "$perl $nytprofhtml --file=$profile_datafile --out=$outdir";
+            my $htmloutdir = "/tmp/$outdir";
+            unlink <$htmloutdir/*>;
+            my $cmd = "$perl $nytprofhtml --file=$profile_datafile --out=$htmloutdir";
             $cmd .= " --open" if $opts{open};
             run_command($cmd);
         }
@@ -341,9 +361,12 @@ sub run_command {
 }
 
 
+# some tests use profile_this() in Devel::NYTProf::Run
 sub run_perl_command {
     my ($cmd, $show_stdout) = @_;
-    run_command("$this_perl $cmd", $show_stdout);
+    local $ENV{PERL5LIB} = $perl5lib;
+    my @perl = perl_command_words(skip_sitecustomize => 1);
+    run_command("@perl $cmd", $show_stdout);
 }
 
 
@@ -368,7 +391,7 @@ sub verify_data {
     SKIP: {
         skip 'Expected profile data does not have VMS paths', 1
             if $^O eq 'VMS' and $test =~ m/test60|test14/i;
-	$profile->normalize_variables;
+        $profile->normalize_variables;
         dump_profile_to_file($profile, $test.'_new', $test.'_newp');
         my @got      = slurp_file($test.'_new'); chomp @got;
         my @expected = slurp_file($test);        chomp @expected;
@@ -407,13 +430,11 @@ sub dump_profile_to_file {
     return;
 }
 
-
 sub diff_files {
     my ($old_file, $new_file, $newp_file) = @_;
 
     # we don't care if this fails, it's just an aid to debug test failures
-    my @opts = split / /, $ENV{NYTPROF_DIFF_OPTS} || '';    # e.g. '-y'
-    @opts = ('-u') unless @opts;
+    my @opts = split / /, $ENV{NYTPROF_DIFF_OPTS} || $diff_opts;    # e.g. '-y'
     system("cmp -s $new_file $newp_file || diff @opts $old_file $new_file 1>&2");
 }
 
@@ -434,7 +455,7 @@ sub verify_csv_report {
     $csvfile =~ s/\.x//;
     $csvfile .= ".p" unless $csvfile =~ /\.p/;
     $csvfile = html_safe_filename($csvfile);
-    $csvfile = "$outdir/${csvfile}-line.csv";
+    $csvfile = "$outdir/${csvfile}-1-line.csv";
     unlink $csvfile;
 
     my $cmd = "$perl $nytprofcsv --file=$profile_datafile --out=$outdir";
@@ -586,4 +607,4 @@ sub count_of_failed_tests {
 
 1;
 
-# vim:ts=8:sw=4
+# vim:ts=8:sw=4:et
